pub struct Dynamic<T>(/* private fields */);Expand description
An instance of a value that provides APIs to observe and react to its contents.
Implementations
sourceimpl<T> Dynamic<T>
impl<T> Dynamic<T>
sourcepub fn downgrade(&self) -> WeakDynamic<T>
pub fn downgrade(&self) -> WeakDynamic<T>
sourcepub fn linked<R, TIntoR, TIntoRResult, RIntoT, RIntoTResult>(
&self,
t_into_r: TIntoR,
r_into_t: RIntoT
) -> Dynamic<R>
pub fn linked<R, TIntoR, TIntoRResult, RIntoT, RIntoTResult>( &self, t_into_r: TIntoR, r_into_t: RIntoT ) -> Dynamic<R>
Returns a new dynamic that has its contents linked with self by the
pair of mapping functions provided.
When the returned dynamic is updated, r_into_t will be invoked. This
function accepts &R and can return T, or Option<T>. If a value is
produced, self will be updated with the new value.
When self is updated, t_into_r will be invoked. This function
accepts &T and can return R or Option<R>. If a value is produced,
the returned dynamic will be updated with the new value.
Panics
This function panics if calling t_into_r with the current contents of
the Dynamic produces a None value. This requirement is only for the
first invocation, and it is guaranteed to occur before this function
returns.
sourcepub fn linked_string(&self) -> Dynamic<String>
pub fn linked_string(&self) -> Dynamic<String>
Creates a linked dynamic containing a String.
When self is updated, ToString::to_string() will be called to
produce a new string value to store in the returned dynamic.
When the returned dynamic is updated, str::parse is called
to produce a new T. If an error is returned, self will not be
updated. Otherwise, self will be updated with the produced value.
sourcepub fn map_ref<R>(&self, map: impl FnOnce(&T) -> R) -> R
pub fn map_ref<R>(&self, map: impl FnOnce(&T) -> R) -> R
Maps the contents with read-only access.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn map_generational<R>(
&self,
map: impl FnOnce(&GenerationalValue<T>) -> R
) -> R
pub fn map_generational<R>( &self, map: impl FnOnce(&GenerationalValue<T>) -> R ) -> R
Maps the contents with read-only access, providing access to the value’s
Generation.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn map_mut<R>(&self, map: impl FnOnce(&mut T) -> R) -> R
pub fn map_mut<R>(&self, map: impl FnOnce(&mut T) -> R) -> R
Maps the contents with exclusive access. Before returning from this function, all observers will be notified that the contents have been updated.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn toggle(&self) -> T
pub fn toggle(&self) -> T
Updates the value to the result of invoking Not on the current
value. This function returns the new value.
sourcepub fn set_source(&self, source: CallbackHandle)
pub fn set_source(&self, source: CallbackHandle)
Sets the current source for this dynamic with source.
A dynamic can have multiple source callbacks.
This ensures that source stays active as long as any clones of self
are alive.
sourcepub fn debounced_every(&self, period: Duration) -> Self
pub fn debounced_every(&self, period: Duration) -> Self
Returns a new dynamic that contains the updated contents of this dynamic
at most once every period.
sourcepub fn debounced_with_delay(&self, period: Duration) -> Self
pub fn debounced_with_delay(&self, period: Duration) -> Self
Returns a new dynamic that contains the updated contents of this dynamic
delayed by period. Each time this value is updated, the delay is
reset.
sourcepub fn map_each_into<U>(&self) -> Dynamic<U>
pub fn map_each_into<U>(&self) -> Dynamic<U>
Returns a new dynamic that is updated using U::from(T.clone()) each
time self is updated.
sourcepub fn map_each_to<U>(&self) -> Dynamic<U>
pub fn map_each_to<U>(&self) -> Dynamic<U>
Returns a new dynamic that is updated using U::from(&T) each
time self is updated.
sourcepub fn for_each<F>(&self, for_each: F) -> CallbackHandle
pub fn for_each<F>(&self, for_each: F) -> CallbackHandle
Attaches for_each to this value so that it is invoked each time the
value’s contents are updated.
sourcepub fn for_each_generational<F>(&self, for_each: F) -> CallbackHandle
pub fn for_each_generational<F>(&self, for_each: F) -> CallbackHandle
Attaches for_each to this value and its Generation so that it is
invoked each time the value’s contents are updated.
sourcepub fn for_each_cloned<F>(&self, for_each: F) -> CallbackHandle
pub fn for_each_cloned<F>(&self, for_each: F) -> CallbackHandle
Attaches for_each to this value so that it is invoked each time the
value’s contents are updated.
sourcepub fn with_for_each<F>(self, for_each: F) -> Self
pub fn with_for_each<F>(self, for_each: F) -> Self
Attaches for_each to this value so that it is invoked each time the
value’s contents are updated. This function returns self.
sourcepub fn map_each<R, F>(&self, map: F) -> Dynamic<R>
pub fn map_each<R, F>(&self, map: F) -> Dynamic<R>
Creates a new dynamic value that contains the result of invoking map
each time this value is changed.
sourcepub fn map_each_generational<R, F>(&self, map: F) -> Dynamic<R>
pub fn map_each_generational<R, F>(&self, map: F) -> Dynamic<R>
Creates a new dynamic value that contains the result of invoking map
each time this value is changed.
sourcepub fn map_each_cloned<R, F>(&self, map: F) -> Dynamic<R>
pub fn map_each_cloned<R, F>(&self, map: F) -> Dynamic<R>
Creates a new dynamic value that contains the result of invoking map
each time this value is changed.
sourcepub fn with_clone<R>(&self, with_clone: impl FnOnce(Self) -> R) -> R
pub fn with_clone<R>(&self, with_clone: impl FnOnce(Self) -> R) -> R
A helper function that invokes with_clone with a clone of self. This
code may produce slightly more readable code.
let value = gooey::value::Dynamic::new(1);
// Using with_clone
value.with_clone(|value| {
std::thread::spawn(move || {
println!("{}", value.get());
})
});
// Using an explicit clone
std::thread::spawn({
let value = value.clone();
move || {
println!("{}", value.get());
}
});
println!("{}", value.get());sourcepub fn get(&self) -> Twhere
T: Clone,
pub fn get(&self) -> Twhere
T: Clone,
Returns a clone of the currently contained value.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn get_tracking_redraw(&self, context: &WidgetContext<'_, '_>) -> Twhere
T: Clone,
pub fn get_tracking_redraw(&self, context: &WidgetContext<'_, '_>) -> Twhere
T: Clone,
Returns a clone of the currently contained value.
context will be invalidated when the value is updated.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn get_tracking_invalidate(&self, context: &WidgetContext<'_, '_>) -> Twhere
T: Clone,
pub fn get_tracking_invalidate(&self, context: &WidgetContext<'_, '_>) -> Twhere
T: Clone,
Returns a clone of the currently contained value.
context will be invalidated when the value is updated.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn take(&self) -> Twhere
T: Default,
pub fn take(&self) -> Twhere
T: Default,
Returns the currently stored value, replacing the current contents with
T::default().
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn take_if_not_default(&self) -> Option<T>
pub fn take_if_not_default(&self) -> Option<T>
Checks if the currently stored value is different than T::default(),
and if so, returns Some(self.take()).
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn replace(&self, new_value: T) -> Option<T>where
T: PartialEq,
pub fn replace(&self, new_value: T) -> Option<T>where
T: PartialEq,
Replaces the contents with new_value, returning the previous contents.
Before returning from this function, all observers will be notified that
the contents have been updated.
If the calling thread has exclusive access to the contents of this
dynamic, this call will return None and the value will not be updated.
If detecting this is important, use Self::try_replace().
sourcepub fn try_replace(&self, new_value: T) -> Result<T, ReplaceError<T>>where
T: PartialEq,
pub fn try_replace(&self, new_value: T) -> Result<T, ReplaceError<T>>where
T: PartialEq,
Replaces the contents with new_value if new_value is different than
the currently stored value. If the value is updated, the previous
contents are returned.
Before returning from this function, all observers will be notified that the contents have been updated.
Errors
ReplaceError::NoChange: Returned whennew_valueis equal to the currently stored value.ReplaceError::Deadlock: Returned when the current thread already has exclusive access to the contents of this dynamic.
sourcepub fn set(&self, new_value: T)where
T: PartialEq,
pub fn set(&self, new_value: T)where
T: PartialEq,
Stores new_value in this dynamic. Before returning from this function,
all observers will be notified that the contents have been updated.
If the calling thread has exclusive access to the contents of this
dynamic, this call will return None and the value will not be updated.
If detecting this is important, use Self::try_replace().
sourcepub fn try_compare_swap(
&self,
expected_current: &T,
new_value: T
) -> Result<T, TryCompareSwapError<T>>
pub fn try_compare_swap( &self, expected_current: &T, new_value: T ) -> Result<T, TryCompareSwapError<T>>
Replaces the current value with new_value if the current value is
equal to expected_current.
Returns Ok with the overwritten value upon success.
Errors
TryCompareSwapError::Deadlock: This operation would result in a thread deadlock.TryCompareSwapError::CurrentValueMismatch: The current value did not matchexpected_current. TheTreturned is a clone of the currently stored value.
sourcepub fn compare_swap(&self, expected_current: &T, new_value: T) -> Result<T, T>
pub fn compare_swap(&self, expected_current: &T, new_value: T) -> Result<T, T>
Replaces the current value with new_value if the current value is
equal to expected_current.
Returns Ok with the overwritten value upon success.
Errors
Returns Err with the currently stored value when expected_current
does not match the currently stored value.
sourcepub fn create_reader(&self) -> DynamicReader<T>
pub fn create_reader(&self) -> DynamicReader<T>
Returns a new reference-based reader for this dynamic value.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn into_reader(self) -> DynamicReader<T>
pub fn into_reader(self) -> DynamicReader<T>
sourcepub fn lock(&self) -> DynamicGuard<'_, T>
pub fn lock(&self) -> DynamicGuard<'_, T>
Returns an exclusive reference to the contents of this dynamic.
This call will block until all other guards for this dynamic have been dropped.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn generation(&self) -> Generation
pub fn generation(&self) -> Generation
Returns the current generation of the value.
Panics
This function panics if this value is already locked by the current thread.
sourcepub fn transition_to(&self, new_value: T) -> DynamicTransition<T>
pub fn transition_to(&self, new_value: T) -> DynamicTransition<T>
Returns a pending transition for this value to new_value.
sourcepub fn new_radio(&self, widget_value: T, label: impl MakeWidget) -> Radio<T>
pub fn new_radio(&self, widget_value: T, label: impl MakeWidget) -> Radio<T>
Returns a new Radio that updates this dynamic to widget_value when
pressed. label is drawn next to the checkbox and is also clickable to
select the radio.
sourcepub fn new_select(&self, widget_value: T, label: impl MakeWidget) -> Select<T>
pub fn new_select(&self, widget_value: T, label: impl MakeWidget) -> Select<T>
Returns a new Select that updates this dynamic to widget_value
when pressed. label is drawn next to the checkbox and is also
clickable to select the widget.
sourcepub fn validate_with<E, Valid>(&self, check: Valid) -> Dynamic<Validation>
pub fn validate_with<E, Valid>(&self, check: Valid) -> Dynamic<Validation>
Validates the contents of this dynamic using the check function,
returning a dynamic that contains the validation status.
sourceimpl Dynamic<WidgetInstance>
impl Dynamic<WidgetInstance>
sourcepub fn into_switcher(self) -> Switcher
pub fn into_switcher(self) -> Switcher
Returns a new Switcher widget whose contents is the value of this
dynamic.